import static org.junit.Assert.*;
import org.junit.Test;

import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

public class JUnitTest {

	final String newLine = System.getProperty("line.separator");
	String pathPrefix = null;
	
	public JUnitTest () {
		pathPrefix = new File("").getAbsolutePath() + "/test/files/";
	}
	
	private static String normalizeLineEnds(String s) {
	    return s.replace("\r\n", "\n").replace('\r', '\n');
	}
	
	private void checkEquality(String testFileName) {		
		try {
			//	
			// Define the input and the output files.
			//
			File inputFile  = new File (pathPrefix + File.separator + testFileName + ".in.txt");
			File outputFile = new File (pathPrefix + File.separator + testFileName + ".out.txt");
			//
			// Read the expected output
			//
			String expectedResult = "";
			BufferedReader buffReader = new BufferedReader(new FileReader(outputFile.getPath()));
	        String line = buffReader.readLine();
	        while (line != null) {
	        	expectedResult += line + '\n';
	            line = buffReader.readLine();
	        }
	        buffReader.close();
	        
			//
			// Define the input and output streams
			// 
			final ByteArrayOutputStream baos = new ByteArrayOutputStream();
			FileInputStream in = new FileInputStream(inputFile);		
			PrintStream out = new PrintStream(baos);			
			//
			// Perform the test
			//		
			ReadAndWrite rw = new ReadAndWrite();
			rw.readAndSolve(in, out);
			//
			// Flush the output and obtain the data
			//
			out.flush();
			out.close();
			String result = baos.toString();
			//
			// Now check for the actual assertion
			//
			assertEquals(normalizeLineEnds(expectedResult), normalizeLineEnds(result));		
		} catch (IOException e) {			
			fail(e.getMessage());
			
		}		
	}
	
	@Test
	public void testTiny() {
		checkEquality("tiny");
	}
	
	@Test
	public void testSmall() {
		checkEquality("small");
	}
	
	@Test
	public void testLarge() {
		// on i7-8650U: reference solution runs in 0.8 s
		// and 1 s on Judge
		checkEquality("large");
	}
	
	@Test
	public void testSingle() {
		AlignDNA ad = new AlignDNA("TAGCAGTTACC", "CTAGAGGTCA");
		assertEquals(8, ad.solve());
	}
}







