/**
 * Using this template, your task is to implement functions denoted by TODO in the comments.
 * 
 * You are given an undirected graph G represented by an adjacency matrix graph,
 * which is symmetric, as seen in tasks during the semester.
 * 
 * Your task is to implement four graph algorithms, see their description
 * by the methods of class Graph.
 * 
 * The Java project also contains tests, in Eclipse you can run them by left-clicking:
 * test/(default package)/JUnitTest.java and selecting Run as -> JUnitTest.java.
 * 
 * The tests will show you if they passed/failed. In case of fail, you can see an exception
 * if the code did not finished, or the difference between your output and expected output.
 * The test names should help you to explain what is being tested. You can also see the content
 * of the tests, for the format of the input, see <code>read</code> method.
 * 
 * The output format is described in the comment of <code>output</code> method.
 */

/**
 * This imports policy applies to all programming assignments and the exam.
 * 
 * You are forbidden to add any other import statements. Make sure that you do not leave any
 * imports commented out, or that your IDE does not add any automatically. You can recognize
 * allowed imports by the <code>// allowed import</code> comment.
 * Calls by the fully qualified name are treated as imports as well. Those are, for example,
 * <code>java.util.Arrays.sort</code>, or <code>java.util.Arrays.binarySearch</code>.
 * You can use data structures we provide to you by imports, by our definitions (even
 * implicitly used classes of packages, such as methods of Object those are inherited with
 * every class declaration, Array, or String), or data structures that you write on your own.
 * Usage of common arrays (<code>type[] variable</code>) is not restricted.
 * 
 * Note that Judge is not enforcing this policy. So if you violate this policy, you may lose the
 * bonus points even if Judge accepts your submission.
 * 
 * The general exceptions out of this policy is the package Math. The exceptions that are specific
 * to a given template are written down by its imports.
 */
import java.util.LinkedList; // allowed import
import java.util.ArrayList; // allowed import
import java.util.Queue; // allowed import
import java.util.Stack; // allowed import
import java.util.StringTokenizer; // allowed import
import java.io.BufferedReader; // allowed import
import java.io.IOException; // allowed import
import java.io.InputStream; // allowed import
import java.io.InputStreamReader; // allowed import
import java.io.PrintStream; // allowed import
import java.lang.Math; // allowed import

class Main {
	public static void main(String[] args) {
		ReadAndWrite rw = new ReadAndWrite();
		rw.readAndSolve(System.in, System.out);
	}
}

class Graph {
	boolean[][] graph; // undirected graph represented by an adjacency matrix
	// the matrix does not contains self-loops (i.e., diagonal is set to false)
	
	// feel free to add any other attributes

	/**
	 * The class is instantiated once for every graph.
	 */
	Graph(boolean[][] g) {
		graph = g;
		// initialize other attributes, if you declared any
	}
	
	/**
	 * TODO: Definition: A graph is k-regular if and only if each vertex
	 * has exactly k neighbors, i.e. every vertex has degree k.
	 * 
	 * 4 points
	 * 
	 * @return the value k if the graph is k-regular for some integer k.
	 * Otherwise, return -1.
	 */
	int isKRegular() {
		return -1;
	}
	
	/**
	 * TODO: test, if the graph contains a triangle, i.e., a cycle of length 3.
	 * 
	 * 4 points
	 * 
	 * @return If the graph contains a triangle, it returns true, otherwise it returns false.
	 */
	boolean hasTriangle() {
		return false;
	}

	/**
	 * TODO: Definition: Given two vertices u and v, their graph distance d(u,v)
	 * is the minimal number of edges that a path connecting u and v has.
	 * If there is no such a path, the distance is infinity, but for programming purposes,
	 * we will define it as -1.
	 * The graph diameter is the maximal distance d(u,v) between any two vertices u, v in G.
	 * If the graph contains at least 2 disconnected components, the diameter is infinity
	 * (i.e., -1 in the code).
	 * 
	 * Your task is to compute the diameter of <code>graph</code>.
	 * 
	 * 10 points
	 * 
	 * @return a disconnected graph (a graph with multiple components) has diameter infinity, return -1.
	 * A graph of a single node has diameter 0.
	 * If the graph is connected, return the maximal graph distance between any two vertices.
	 */
	int getGraphDiameter() {
		return -1;
	}
	
	/**
	 * This function is not tested by Judge or JUnitTest, it serves
	 * only for debugging - visualization by testSingle. Find the
	 * description how to use it in JUnitTest file.
	 * 
	 * @return string representation of color for given vertex
	 * <code>i</code>.
	 */
	String getNodeColor(int i) {
		switch (i % 3) {
		case 1:
			return "blue";
		case 2:
			return "red";

		default:
			return "white";
		}
	}
}

class Tile {
	int a;
	int b;
	
	Tile(int aa, int bb) {
		a = aa;
		b = bb;
	}

	/**
	 * TODO: decide if these <code>tiles</code> can form a single sequence such that
	 * two adjacent tiles are connected by the same number.
	 * 
	 * You can use the provided function <code>containsEulerianWalk()</code> below.
	 * 
	 * The tile numbers are not arbitrary large, the largest tile number is about
	 * the total of distinct tile numbers. However, there can be tile with number,
	 * e.g., 2, even without number 1 appearing on any tile.
	 * 
	 * 
	 * 4 points
	 * 
	 * @return If the tiles can form the described sequence, return true, else return false.
	 */
	boolean dominoSequence(Tile[] tiles) {
		return false;
	}

	/**
	 * For the given undirected graph g, given as before by an adjacency matrix,
	 * determines if it contains an Eulerian walk.

	 * The input is symmetrical adjacancy matrix (with true for existing edges, and false else).
	 * 
	 * @return If g contains Eulerian walk, returns true, otherwise returns false.
	 */
	boolean containsEulerianWalk(boolean[][] g) {
		// single non-trivial component?
		int[] clusters = new int[g.length];
		for (int i = 0; i < clusters.length; i++) {
			clusters[i] = 0;
		}
		int cl_counter = 1;
		for (int i = 0; i < g.length; i++) {
			if (clusters[i] == 0) {
				clusters[i] = cl_counter++;
				dfs(g, clusters, i);
			}
		}
		int[] cl_sizes = new int[cl_counter];
		for (int i = 0; i < cl_sizes.length; i++) {
			cl_sizes[i] = 0;
		}
		
		for (int i = 0; i < g.length; i++) {
			cl_sizes[clusters[i]]++;
		}
		
		int nontrivial_components = 0;
		for (int i = 0; i < cl_sizes.length; i++) {
			if (cl_sizes[i] > 1) {
				++nontrivial_components;
			}
		}
		
		if (nontrivial_components > 1) {
			return false;
		}
		
		// connectivity
		int odd_vertices = 0;
		for (int i = 0; i < g.length; i++) {
			int deg = 0;
			for (int j = 0; j < i; j++) {
				if (g[i][j]) {
					++deg;
				}
			}
			
			if (deg % 2 == 1) {
				++odd_vertices;
			}
			if (odd_vertices > 2) {
				return false;
			}
		}
		
		return true;
	}

	void dfs(boolean[][] g, int[] cluster, int cur) {
		for (int i = 0; i < g.length; i++) {
			if ((g[cur][i] || g[i][cur]) && cluster[i] == 0) {
				cluster[i] = cluster[cur];
				dfs(g, cluster, i);
			}
		}
	}

}
///////////////////////////////////////////////////////////////////////
// DO NOT MODIFY THE FOLLOWING CODE, YOU CAN COMPLETELY IGNORE IT
// WE MAY USE LANGUAGE CONSTRUCTS THAT YOU MAY HAVE NOT SEEN SO FAR
///////////////////////////////////////////////////////////////////////

class ReadAndWrite {
	/**
	 * Parses input in form:
	 * <mode: 'd'=domino, 'k'=KReg, 't'=triangleFree, 'g'=diameter>  
	 * <number of elements = n> 
	 * <n graphs in graph6 format>
	 * 
	 */
	void readAndSolve(InputStream in, PrintStream out) {
		FastReader s = new FastReader(in);
		char inMode = s.nextChar();
		
		int n = s.nextInt();

		for (int i = 0; i < n; i++) {
			if (inMode == 'd') {
				Tile t = new Tile(0, 0);
				int m = s.nextInt();
				Tile[] tiles = new Tile[m];
				for (int j = 0; j < m; j++) {
					tiles[j] = new Tile(s.nextInt(), s.nextInt());
				}
				out.println(t.dominoSequence(tiles) ? 1 : 0);
				
			} else {
				String line = s.next();
				
				Graph g = new Graph(parseGraph(line));
	
				switch (inMode) {
				case 'k':
					out.println(g.isKRegular());
					break;
				case 't':
					out.println(g.hasTriangle() ? 1 : 0);
					break;
				case 'g':
					out.println(g.getGraphDiameter());
					break;
	
				default:
					break;
				}
			}
		}
	}
	
	static boolean[][] parseGraph(String line) {
        ByteReader6 br6 = new ByteReader6(line);
        int n = br6.get_number();

        boolean[][] matrix = new boolean[n][n];
        for (int i = 1; i < n; ++i) {
            for (int j = 1; j < n; ++j) {
            	matrix[i][j] = false;
            }
        }

        for (int j = 1; j < n; ++j) {
            for (int i = 0; i < j; ++i) {
                int e = br6.get_bit();
                if (e != 0) {
                	matrix[i][j] = true;
                	matrix[j][i] = true;
                }
            }
        }
        return matrix;
	}
}

/**
 * Ignore this class please. It is used for input parsing. Source:
 * https://www.geeksforgeeks.org/fast-io-in-java-in-competitive-programming/
 */
class FastReader {
	BufferedReader br;
	StringTokenizer st;

	FastReader(InputStream in) {
		br = new BufferedReader(new InputStreamReader(in));
	}

	String next() {
		while (st == null || !st.hasMoreElements()) {
			try {
				st = new StringTokenizer(br.readLine());
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return st.nextToken();
	}

	int nextInt() {
		return Integer.parseInt(next());
	}

	char nextChar() {
		return next().charAt(0);
	}

	String nextLine() {
		String str = "";
		try {
			str = br.readLine();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return str;
	}
}

/**
 * Ignore this class please. It is used for input parsing. Source:
 * https://github.com/bingmann/BispanningGame/blob/master/src/net/panthema/BispanningGame/Graph6.java
 */
class ByteReader6
{
    private byte[] mBytes;
    private int mSize, mPos, mBit;

    public ByteReader6(String s6) {
        mBytes = s6.getBytes();
        mSize = s6.length();
        mPos = mBit = 0;
    }

    // ! whether k bits are available
    boolean have_bits(int k) {
        return (mPos + (mBit + k - 1) / 6) < mSize;
    }

    // ! return the next integer encoded in graph6
    // fixed by Karel Kubicek
    int get_number() {
        assert (mPos == 0);
        assert (mSize > 0);
        
    	int n = 0;
    	int nBytes;
    	byte c;
    	
    	if (mBytes[0] < 126) { // 0 <= n <= 62
    		nBytes = 1;
    	} else if (mBytes[1] < 126) { // 63 <= n <= 258047
    		nBytes = 3;
    		++mPos;
    	} else { // 258048 <= n <= 68719476735
    		nBytes = 6;
    		mPos += 2;
    	}

        
    	for (int i = 0; i < nBytes; ++i) {
    		n *= 64;
        	c = mBytes[mPos];
        	++mPos;
        	
        	assert (c >= 63);
	        c -= 63;
	        assert (c <= 63);
	        n += c;
    	}
        return n;
        
        // ignore that for 2^31 <= n <= 2^36
        // it does not fit into integer, as we cannot allocate so much memory in Java anyway
    }

    // ! return the next bit encoded in graph6
    int get_bit() {
        assert (mPos < mSize);

        byte c = mBytes[mPos];
        assert (c >= 63);
        c -= 63;
        c >>= (5 - mBit);

        mBit++;
        if (mBit == 6) {
            mPos++;
            mBit = 0;
        }

        return (c & 0x01);
    }

    // ! return the next bits as an integer
    int get_bits(int k) {
        int v = 0;

        for (int i = 0; i < k; ++i) {
            v *= 2;
            v += get_bit();
        }

        return v;
    }
}
