import static org.junit.Assert.*;
import org.junit.Test;

import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;

public class JUnitTest {

	final String newLine = System.getProperty("line.separator");
	String pathPrefix = null;
	
	public JUnitTest () {
		pathPrefix = new File("").getAbsolutePath() + "/test/files/";
	}
	
	private static String normalizeLineEnds(String s) {
	    return s.replace("\r\n", "\n").replace('\r', '\n');
	}
	
	private void checkEquality(String testFileName) {		
		try {
			//	
			// Define the input and the output files.
			//
			File inputFile  = new File (pathPrefix + File.separator + testFileName + ".in.txt");
			File outputFile = new File (pathPrefix + File.separator + testFileName + ".out.txt");
			//
			// Read the expected output
			//
			String expectedResult = "";
			BufferedReader buffReader = new BufferedReader(new FileReader(outputFile.getPath()));
	        String line = buffReader.readLine();
	        while (line != null) {
	        	expectedResult += line + '\n';
	            line = buffReader.readLine();
	        }
	        buffReader.close();
	        
			//
			// Define the input and output streams
			// 
			final ByteArrayOutputStream baos = new ByteArrayOutputStream();
			FileInputStream in = new FileInputStream(inputFile);		
			PrintStream out = new PrintStream(baos);			
			//
			// Perform the test
			//		
			ReadAndWrite rw = new ReadAndWrite();
			rw.readAndSolve(in, out);
			//
			// Flush the output and obtain the data
			//
			out.flush();
			out.close();
			String result = baos.toString();
			//
			// Now check for the actual assertion
			//
			assertEquals(normalizeLineEnds(expectedResult), normalizeLineEnds(result));		
		} catch (IOException e) {			
			fail(e.getMessage());
			
		}		
	}
	
	@Test
	public void testTinyKRegular() {
		checkEquality("tiny_k");
	}
	
	@Test
	public void testTinyHasTriangle() {
		checkEquality("tiny_t");
	}
	
	@Test
	public void testTinyDiameter() {
		checkEquality("tiny_g");
	}
	
	@Test
	public void testTinyDomino() {
		checkEquality("tiny_d");
	}
	
	@Test
	public void testSmallKRegular() {
		checkEquality("small_k");
	}
	
	@Test
	public void testSmallHasTriangle() {
		checkEquality("small_t");
	}
	
	@Test
	public void testSmallDiameter() {
		checkEquality("small_g");
	}
	
	@Test
	public void testSmallDomino() {
		checkEquality("small_d");
	}
	
	@Test
	public void testLargeKRegular() {
		// reference solution runs in 3.7 s on i7-8650U
		// and 0.2 s on Judge
		checkEquality("large_k");
	}
	
	@Test
	public void testLargeHasTriangle() {
		// reference solution runs in 2.8 s on i7-8650U
		// and 1 s on Judge
		checkEquality("large_t");
	}
	
	@Test
	public void testLargeDiameter() {
		// reference solution runs in 1.9 s on i7-8650U
		// and 1 s on Judge
		checkEquality("large_g");
	}
	
	@Test
	public void testLargeDomino() {
		// reference solution runs in 0.1 s on i7-8650U
		// and 1 s on Judge
		checkEquality("large_d");
	}
	
	@Test
	public void testSingle() {
		/**
		 * Note that this test will always pass.
		 * Use this test to debug a single wrong graph.
		 * 
		 * Copy the line representing some graph from the input to string
		 * <code>stringGraphToShow</code>. The test will run your method for checking
		 * if the graph is two colorable, and it will generate GraphViz format
		 * representation of the input. You can find this graphviz file in the directory
		 * with test files, as graph.dot.
		 * 
		 * To visualize the file, use either some online tool for graphviz visualization:
		 * e.g., http://www.webgraphviz.com/, or install a local viewer:
		 * https://www.graphviz.org/download/
		 */
		char mode = 'k';
		String stringGraphToShow = "I????????";
		FastReader s = new FastReader(new ByteArrayInputStream(stringGraphToShow.getBytes()));
		
		if (mode == 'd') {
			Tile t = new Tile(0, 0);
			int m = s.nextInt();
			Tile[] tiles = new Tile[m];
			for (int j = 0; j < m; j++) {
				tiles[j] = new Tile(s.nextInt(), s.nextInt());
			}
			System.out.println(t.dominoSequence(tiles));
			
		} else {
			String line = s.next();
			
			Graph g = new Graph(ReadAndWrite.parseGraph(line));

			switch (mode) {
			case 'k':
				System.out.println(g.isKRegular());
				break;
			case 't':
				System.out.println(g.hasTriangle());
				break;
			case 'g':
				System.out.println(g.getGraphDiameter());
				break;

			default:
				break;
			}

			String filename = pathPrefix + "/graph.dot"; 
			
			try {
		    	PrintWriter writer = new PrintWriter(filename, "UTF-8");
				
		    	writer.write("graph G {\n");
		    	
		    	writer.write("{\n node [style=filled]\n");
				for (int i = 0; i < g.graph.length; ++i) {
					writer.write("\"" + i + "\" [fillcolor=" + g.getNodeColor(i) + "]\n");
				}
		    	writer.write("}\n");
		    	
		    	
				for (int i = 0; i < g.graph.length; ++i) {
					for (int j = i+1; j < g.graph.length; ++j) {
						if (g.graph[i][j]) {
							writer.write("\"" + i + "\" -- \"" + j + "\"\n");
						}
					}
				}
				writer.write("}\n");
				
		        writer.close();
		    } catch (FileNotFoundException e) {
				// Auto-generated catch block
				e.printStackTrace();
			} catch (UnsupportedEncodingException e) {
				// Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
}







